@@ -9,7 +9,7 @@ sequences in Python literals.
 
 usage: standardfonts.py [dec|hex|oct]
 """
-__version__=''' $Id: stdfonts.py 3959 2012-09-27 14:39:39Z robin $ '''
+__version__=''' $Id$ '''
 import sys
 from reportlab.pdfbase import pdfmetrics
 from reportlab.pdfgen import canvas
@@ -64,10 +64,10 @@ if __name__ == '__main__':
     if len(sys.argv)==2:
         mode = string.lower(sys.argv[1])
         if mode not in ['dec','oct','hex']:
-            print __doc__
+            print(__doc__)
 
     elif len(sys.argv) == 1:
         mode = 'dec'
         run(mode)
     else:
-        print __doc__
+        print(__doc__)
