@@ -49,7 +49,7 @@ includes converters for all the common encodings, including Asian ones.
 
 
 
-disc(u"""
+disc("""
 If your data is not encoded as UTF8, you will get a UnicodeDecodeError as
 soon as you feed in a non-ASCII character.  For example, this snippet below is
 attempting to read in and print a series of names, including one with a French
@@ -57,7 +57,7 @@ accent:  ^Marc-Andr\u00e9 Lemburg^.  The standard error is quite helpful and tel
 what character it doesn't like:
 """)
 
-eg(u"""
+eg("""
 >>> from reportlab.pdfgen.canvas import Canvas
 >>> c = Canvas('temp.pdf')
 >>> y = 700
@@ -85,19 +85,22 @@ eg("""
 """)
 
 
-heading2("Changing the built-in fonts output encoding")
+heading2("Automatic output font substitution")
 
 disc("""
 There are still a number of places in the code, including the rl_config
-defaultEncoding parameter, and arguments passed to various Font constructors.
-These generally relate to the OUTPUT encoding used when we write data in the font
-file. This affects which characters are actually available in the font if
-you are using Type 1 fonts, since only 256 glyphs can be available at
-one time. Unless you have a very specific need for
-MacRoman or MacExpert encoding characters, we advise you to ignore
-this.  By default the standard fonts (Helvetica, Courier, Times Roman)
-will offer the glyphs available in Latin-1.  If you try to print a non-Latin-1
-character using the built-in Helvetica, you'll see a rectangle or blob.
+defaultEncoding parameter, and arguments passed to various Font constructors,
+which refer to encodings.  These were useful in the past when people needed to
+use glyphs in the Symbol and ZapfDingbats fonts which are supported by PDF
+viewing devices.
+
+By default the standard fonts (Helvetica, Courier, Times Roman)
+will offer the glyphs available in Latin-1.  However, if our engine detects
+a character not in the font, it will attempt to switch to Symbol or ZapfDingbats to
+display these.   For example, if you include the Unicode character for a pair of 
+right-facing scissors, \\u2702, in a call to ^drawString^, you should see them (there is
+an example in ^test_pdfgen_general.py/pdf^).  It is not
+necessary to switch fonts in your code.
 
 """)
 
@@ -324,7 +327,7 @@ from reportlab.pdfbase.pdfmetrics import registerFontFamily
 registerFontFamily('Vera',normal='Vera',bold='VeraBd',italic='VeraIt',boldItalic='VeraBI')
 
 disc("""Before using the TT Fonts in Platypus we should add a mapping from the family name to the
-individual font names that describe the behaviour under the $<b>$ and $<i>$ attributes.""")
+individual font names that describe the behaviour under the $&lt;b&gt;$ and $&lt;i&gt;$ attributes.""")
 
 eg("""
 from reportlab.pdfbase.pdfmetrics import registerFontFamily
