@@ -1,4 +1,4 @@
-#Copyright ReportLab Europe Ltd. 2000-2004
+#Copyright ReportLab Europe Ltd. 2000-2012
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/docs/userguide/ch5_paragraphs.py
 from tools.docco.rl_doc_utils import *
@@ -195,7 +195,7 @@ def getAttrs(A):
         else:
             S[a] = "%s, %s" %(S[a],k)
 
-    K = S.keys()
+    K = list(S.keys())
     K.sort()
     D=[('Attribute','Synonyms')]
     for k in K:
@@ -371,7 +371,7 @@ disc("""The &lt;bullet&gt; tag is only allowed once in a given paragraph and its
 overrides the implied bullet style and ^bulletText^ specified in the  ^Paragraph^
 creation.
 """)
-parabox("""<bullet>\xe2\x80\xa2</bullet>this is a bullet point.  Spam
+parabox("""<bullet>\u2022</bullet>this is a bullet point.  Spam
 spam spam spam spam spam spam spam spam spam spam spam
 spam spam spam spam spam spam spam spam spam spam """,
         styleSheet['Bullet'],
