@@ -1,15 +1,15 @@
 #Copyright ReportLab Europe Ltd. 2000-2012
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/graphics/charts/textlabels.py
-__version__=''' $Id: textlabels.py 3959 2012-09-27 14:39:39Z robin $ '''
+__version__=''' $Id$ '''
 import string
 
 from reportlab.lib import colors
 from reportlab.lib.utils import simpleSplit, _simpleSplit
 from reportlab.lib.validators import isNumber, isNumberOrNone, OneOf, isColorOrNone, isString, \
-        isTextAnchor, isBoxAnchor, isBoolean, NoneOr, isInstanceOf, isNoneOrString
+        isTextAnchor, isBoxAnchor, isBoolean, NoneOr, isInstanceOf, isNoneOrString, isNoneOrCallable
 from reportlab.lib.attrmap import *
-from reportlab.pdfbase.pdfmetrics import stringWidth
+from reportlab.pdfbase.pdfmetrics import stringWidth, getAscentDescent
 from reportlab.graphics.shapes import Drawing, Group, Circle, Rect, String, STATE_DEFAULTS
 from reportlab.graphics.shapes import _PATH_OP_ARG_COUNT, _PATH_OP_NAMES, definePath
 from reportlab.graphics.widgetbase import Widget, PropHolder
@@ -35,14 +35,14 @@ def _processGlyph(G, truncate=1, pathReverse=0):
             if O[0]=='moveToClosed':
                 O = O[1:]
                 if pathReverse:
-                    for i in xrange(0,len(P),2):
+                    for i in range(0,len(P),2):
                         P[i+1], P[i] = P[i:i+2]
                     P.reverse()
                     O.reverse()
                 O.insert(0,'moveTo')
                 O.append('closePath')
             i = 0
-            if truncate: P = map(_pathNumTrunc,P)
+            if truncate: P = list(map(_pathNumTrunc,P))
             for o in O:
                 j = i + _PATH_OP_ARG_COUNT[_PATH_OP_NAMES.index(o)]
                 if o=='closePath':
@@ -118,6 +118,8 @@ class Label(Widget):
         leftPadding = AttrMapValue(isNumber,desc='padding at left of box'),
         rightPadding = AttrMapValue(isNumber,desc='padding at right of box'),
         bottomPadding = AttrMapValue(isNumber,desc='padding at bottom of box'),
+        useAscentDescent = AttrMapValue(isBoolean,desc="If True then the font's Ascent & Descent will be used to compute default heights and baseline."),
+        customDrawChanger = AttrMapValue(isNoneOrCallable,desc="An instance of CustomDrawChanger to modify the behavior at draw time", _advancedUsage=1),
         )
 
     def __init__(self,**kw):
@@ -149,6 +151,7 @@ class Label(Widget):
                 strokeWidth = 0.1,
                 textAnchor = 'start',
                 visible = 1,
+                useAscentDescent = False,
                 )
 
     def setText(self, text):
@@ -213,7 +216,18 @@ class Label(Widget):
                 self._width += max(self._lineWidths)
         else:
             self._width = self.width
-        self._height = self.height or ((self.leading or 1.2*self.fontSize) * len(self._lines)+topPadding+bottomPadding)
+        if self.useAscentDescent:
+            self._ascent, self._descent = getAscentDescent(self.fontName,self.fontSize)
+            self._baselineRatio = self._ascent/(self._ascent-self._descent)
+        else:
+            self._baselineRatio = 1/1.2
+        if self.leading:
+            self._leading = self.leading
+        elif self.useAscentDescent:
+            self._leading = self._ascent - self._descent
+        else:
+            self._leading = self.fontSize*1.2
+        self._height = self.height or (self._leading*len(self._lines) + topPadding + bottomPadding)
         self._ewidth = (self._width-leftPadding-rightPadding)
         self._eheight = (self._height-topPadding-bottomPadding)
         boxAnchor = self._getBoxAnchor()
@@ -239,7 +253,7 @@ class Label(Widget):
         if ta=='boxauto': ta = _BA2TA[self._getBoxAnchor()]
         return ta
 
-    def draw(self):
+    def _rawDraw(self):
         _text = self._text
         self._text = _text or ''
         self.computeSize()
@@ -248,7 +262,7 @@ class Label(Widget):
         g.translate(self.x + self.dx, self.y + self.dy)
         g.rotate(self.angle)
 
-        y = self._top - self.fontSize
+        y = self._top - self._leading*self._baselineRatio
         textAnchor = self._getTextAnchor()
         if textAnchor == 'start':
             x = self._left
@@ -270,7 +284,8 @@ class Label(Widget):
                         )
 
         fillColor, fontName, fontSize = self.fillColor, self.fontName, self.fontSize
-        strokeColor, strokeWidth, leading = self.strokeColor, self.strokeWidth, (self.leading or 1.2*fontSize)
+        strokeColor, strokeWidth, leading = self.strokeColor, self.strokeWidth, self._leading
+        svgAttrs=getattr(self,'_svgAttrs',{})
         if strokeColor:
             for line in self._lines:
                 s = _text2Path(line, x, y, fontName, fontSize, textAnchor)
@@ -278,19 +293,30 @@ class Label(Widget):
                 s.strokeColor = strokeColor
                 s.strokeWidth = strokeWidth
                 g.add(s)
-                y = y - leading
+                y -= leading
         else:
             for line in self._lines:
-                s = String(x, y, line)
+                s = String(x, y, line, _svgAttrs=svgAttrs)
                 s.textAnchor = textAnchor
                 s.fontName = fontName
                 s.fontSize = fontSize
                 s.fillColor = fillColor
                 g.add(s)
-                y = y - leading
+                y -= leading
 
         return g
 
+    def draw(self):
+        customDrawChanger = getattr(self,'customDrawChanger',None)
+        if customDrawChanger:
+            customDrawChanger(True,self)
+            try:
+                return self._rawDraw()
+            finally:
+                customDrawChanger(False,self)
+        else:
+            return self._rawDraw()
+
 class LabelDecorator:
     _attrMap = AttrMap(
         x = AttrMapValue(isNumberOrNone,desc=''),
@@ -375,12 +401,31 @@ class LabelOffset(PropHolder):
 
 NoneOrInstanceOfLabelOffset=NoneOr(isInstanceOf(LabelOffset))
 
-class BarChartLabel(Label):
+class PMVLabel(Label):
+    _attrMap = AttrMap(
+        BASE=Label,
+        )
+
+    def __init__(self):
+        Label.__init__(self)
+        self._pmv = 0
+
+    def _getBoxAnchor(self):
+        a = Label._getBoxAnchor(self)
+        if self._pmv<0: a = {'nw':'se','n':'s','ne':'sw','w':'e','c':'c','e':'w','sw':'ne','s':'n','se':'nw'}[a]
+        return a
+
+    def _getTextAnchor(self):
+        a = Label._getTextAnchor(self)
+        if self._pmv<0: a = {'start':'end', 'middle':'middle', 'end':'start'}[a]
+        return a
+
+class BarChartLabel(PMVLabel):
     """
     An extended Label allowing for nudging, lines visibility etc
     """
     _attrMap = AttrMap(
-        BASE=Label,
+        BASE=PMVLabel,
         lineStrokeWidth = AttrMapValue(isNumberOrNone, desc="Non-zero for a drawn line"),
         lineStrokeColor = AttrMapValue(isColorOrNone, desc="Color for a drawn line"),
         fixedEnd = AttrMapValue(NoneOrInstanceOfLabelOffset, desc="None or fixed draw ends +/-"),
@@ -389,22 +434,11 @@ class BarChartLabel(Label):
         )
 
     def __init__(self):
-        Label.__init__(self)
+        PMVLabel.__init__(self)
         self.lineStrokeWidth = 0
         self.lineStrokeColor = None
-        self.nudge = 0
         self.fixedStart = self.fixedEnd = None
-        self._pmv = 0
-
-    def _getBoxAnchor(self):
-        a = self.boxAnchor
-        if self._pmv<0: a = {'nw':'se','n':'s','ne':'sw','w':'e','c':'c','e':'w','sw':'ne','s':'n','se':'nw'}[a]
-        return a
-
-    def _getTextAnchor(self):
-        a = self.textAnchor
-        if self._pmv<0: a = {'start':'end', 'middle':'middle', 'end':'start'}[a]
-        return a
+        self.nudge = 0
 
 class NA_Label(BarChartLabel):
     """
@@ -418,3 +452,15 @@ class NA_Label(BarChartLabel):
         BarChartLabel.__init__(self)
         self.text = 'n/a'
 NoneOrInstanceOfNA_Label=NoneOr(isInstanceOf(NA_Label))
+
+from reportlab.graphics.charts.utils import CustomDrawChanger
+class RedNegativeChanger(CustomDrawChanger):
+    def __init__(self,fillColor=colors.red):
+        CustomDrawChanger.__init__(self)
+        self.fillColor = fillColor
+    def _changer(self,obj):
+        R = {}
+        if obj._text.startswith('-'):
+            R['fillColor'] = obj.fillColor
+            obj.fillColor = self.fillColor
+        return R
