@@ -1,11 +1,11 @@
 #Copyright ReportLab Europe Ltd. 2000-2012
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/lib/colors.py
-__version__=''' $Id: colors.py 3959 2012-09-27 14:39:39Z robin $ '''
+__version__=''' $Id$ '''
 __doc__='''Defines standard colour-handling classes and colour names.
 
 We define standard classes to hold colours in two models:  RGB and CMYK.
-These can be constructed from several popular formats.  We also include
+rhese can be constructed from several popular formats.  We also include
 
 - pre-built colour objects for the HTML standard colours
 
@@ -39,8 +39,11 @@ Traceback (most recent call last):
     ....
 ValueError: css color 'pcmyka(100,0,0,0)' has wrong number of components
 '''
-import math, re
-from reportlab.lib.utils import fp_str
+import math, re, functools
+from reportlab import isPy3
+from reportlab.lib.rl_accel import fp_str
+from reportlab.lib.utils import asNative, isStr
+import collections
 
 class Color:
     """This class is used to represent color.  Components red, green, blue
@@ -56,25 +59,38 @@ class Color:
     def __repr__(self):
         return "Color(%s)" % fp_str(*(self.red, self.green, self.blue,self.alpha)).replace(' ',',')
 
-    def __hash__(self):
-        return hash((self.red, self.green, self.blue, self.alpha))
-
-    def __cmp__(self,other):
+    @property
+    def __key__(self):
         '''simple comparison by component; cmyk != color ever
         >>> cmp(Color(0,0,0),None)
         -1
         >>> cmp(Color(0,0,0),black)
         0
         >>> cmp(Color(0,0,0),CMYKColor(0,0,0,1)),Color(0,0,0).rgba()==CMYKColor(0,0,0,1).rgba()
-        (-1, True)
+        (1, True)
         '''
-        if isinstance(other,CMYKColor) or not isinstance(other,Color): return -1
+        return self.red, self.green, self.blue, self.alpha
+
+    def __hash__(self):
+        return hash(self.__key__)
+
+    def __comparable__(self,other):
+        return not isinstance(other,CMYKColor) and isinstance(other,Color)
+
+    def __lt__(self,other):
+        if not self.__comparable__(other): return True
         try:
-            return cmp((self.red, self.green, self.blue, self.alpha),
-                    (other.red, other.green, other.blue, other.alpha))
+            return self.__key__ < other.__key__
         except:
-            return -1
-        return 0
+            pass
+        return True
+
+    def __eq__(self,other):
+        if not self.__comparable__(other): return False
+        try:
+            return self.__key__ == other.__key__
+        except:
+            return False
 
     def rgb(self):
         "Returns a three-tuple of components"
@@ -85,10 +101,10 @@ class Color:
         return (self.red, self.green, self.blue, self.alpha)
 
     def bitmap_rgb(self):
-        return tuple(map(lambda x: int(x*255)&255, self.rgb()))
+        return tuple([int(x*255)&255 for x in self.rgb()])
 
     def bitmap_rgba(self):
-        return tuple(map(lambda x: int(x*255)&255, self.rgba()))
+        return tuple([int(x*255)&255 for x in self.rgba()])
 
     def hexval(self):
         return '0x%02x%02x%02x' % self.bitmap_rgb()
@@ -118,7 +134,7 @@ class Color:
 
     def _lookupName(self,D={}):
         if not D:
-            for n,v in getAllNamedColors().iteritems():
+            for n,v in getAllNamedColors().items():
                 if not isinstance(v,CMYKColor):
                     t = v.red,v.green,v.blue
                     if t in D:
@@ -127,6 +143,10 @@ class Color:
         t = self.red,self.green,self.blue
         return t in D and D[t] or None
 
+    @property
+    def normalizedAlpha(self):
+        return self.alpha
+if isPy3: Color = functools.total_ordering(Color)
 
 class CMYKColor(Color):
     """This represents colors using the CMYK (cyan, magenta, yellow, black)
@@ -186,14 +206,12 @@ class CMYKColor(Color):
         *NB* note this dosen't reach density zero'''
         scale = self._scale
         dd = scale/float(n)
-        L = [self.clone(density=scale - i*dd) for i in xrange(n)]
+        L = [self.clone(density=scale - i*dd) for i in range(n)]
         if reverse: L.reverse()
         return L
 
-    def __hash__(self):
-        return hash( (self.cyan, self.magenta, self.yellow, self.black, self.density, self.spotName, self.alpha) )
-
-    def __cmp__(self,other):
+    @property
+    def __key__(self):
         """obvious way to compare colours
         Comparing across the two color models is of limited use.
         >>> cmp(CMYKColor(0,0,0,1),None)
@@ -205,14 +223,10 @@ class CMYKColor(Color):
         >>> cmp(CMYKColor(0,0,0,1),Color(0,0,1)),Color(0,0,0).rgba()==CMYKColor(0,0,0,1).rgba()
         (-1, True)
         """
-        if not isinstance(other, CMYKColor): return -1
-        try:
-            return cmp(
-                (self.cyan, self.magenta, self.yellow, self.black, self.density, self.alpha, self.spotName),
-                (other.cyan, other.magenta, other.yellow, other.black, other.density, other.alpha, other.spotName))
-        except: # or just return 'not equal' if not a color
-            return -1
-        return 0
+        return self.cyan, self.magenta, self.yellow, self.black, self.density, self.spotName, self.alpha
+
+    def __comparable__(self,other):
+        return isinstance(other,CMYKColor)
 
     def cmyk(self):
         "Returns a tuple of four color components - syntactic sugar"
@@ -228,7 +242,7 @@ class CMYKColor(Color):
 
     def _lookupName(self,D={}):
         if not D:
-            for n,v in getAllNamedColors().iteritems():
+            for n,v in getAllNamedColors().items():
                 if isinstance(v,CMYKColor):
                     t = v.cyan,v.magenta,v.yellow,v.black
                     if t in D:
@@ -237,6 +251,10 @@ class CMYKColor(Color):
         t = self.cyan,self.magenta,self.yellow,self.black
         return t in D and D[t] or None
 
+    @property
+    def normalizedAlpha(self):
+        return self.alpha*self._scale
+
 class PCMYKColor(CMYKColor):
     '''100 based CMYKColor with density and a spotName; just like Rimas uses'''
     _scale = 100.
@@ -307,13 +325,13 @@ def color2bw(colorRGB):
     bwColorRGB = Color(n, n, n, a)
     return bwColorRGB
 
-def HexColor(val, htmlOnly=False, alpha=False):
+def HexColor(val, htmlOnly=False, hasAlpha=False):
     """This function converts a hex string, or an actual integer number,
     into the corresponding color.  E.g., in "#AABBCC" or 0xAABBCC,
     AA is the red, BB is the green, and CC is the blue (00-FF).
 
     An alpha value can also be given in the form #AABBCCDD or 0xAABBCCDD where
-    DD is the alpha value.
+    DD is the alpha value if hasAlpha is True.
 
     For completeness I assume that #aabbcc or 0xaabbcc are hex numbers
     otherwise a pure integer is converted as decimal rgb.  If htmlOnly is true,
@@ -345,7 +363,7 @@ def HexColor(val, htmlOnly=False, alpha=False):
 
     """ #" for emacs
 
-    if isinstance(val,basestring):
+    if isinstance(val,str):
         b = 10
         if val[:1] == '#':
             val = val[1:]
@@ -361,8 +379,8 @@ def HexColor(val, htmlOnly=False, alpha=False):
                 if len(val) == 8:
                     alpha = True
         val = int(val,b)
-    if alpha:
-        return Color((val>>24)&0xFF/255.0,((val>>16)&0xFF)/255.0,((val>>8)&0xFF)/255.0,(val&0xFF)/255.0)
+    if hasAlpha:
+        return Color(((val>>24)&0xFF)/255.0,((val>>16)&0xFF)/255.0,((val>>8)&0xFF)/255.0,(val&0xFF)/255.0)
     return Color(((val>>16)&0xFF)/255.0,((val>>8)&0xFF)/255.0,(val&0xFF)/255.0)
 
 def linearlyInterpolatedColor(c0, c1, x0, x1, x):
@@ -377,7 +395,7 @@ def linearlyInterpolatedColor(c0, c1, x0, x1, x):
     if x1<x0:
         x0,x1,c0,c1 = x1,x0,c1,c0 # normalized so x1>x0
     if x<x0-1e-8 or x>x1+1e-8: # fudge factor for numerical problems
-        raise ValueError, "Can't interpolate: x=%f is not between %f and %f!" % (x,x0,x1)
+        raise ValueError("Can't interpolate: x=%f is not between %f and %f!" % (x,x0,x1))
     if x<=x0:
         return c0
     elif x>=x1:
@@ -473,7 +491,7 @@ def linearlyInterpolatedColor(c0, c1, x0, x1, x):
             a = c0.alpha+x*(c1.alpha - c0.alpha)/dx
             return PCMYKColor(c*100,m*100,y*100,k*100, density=d*100, alpha=a*100)
     else:
-        raise ValueError, "Can't interpolate: Unknown color class %s!" % cname
+        raise ValueError("Can't interpolate: Unknown color class %s!" % cname)
 
 def obj_R_G_B(c):
     '''attempt to convert an object to (red,green,blue)'''
@@ -695,9 +713,9 @@ def getAllNamedColors():
     # uses a singleton for efficiency
     global _namedColors
     if _namedColors is not None: return _namedColors
-    import colors
+    from reportlab.lib import colors
     _namedColors = {}
-    for (name, value) in colors.__dict__.items():
+    for name, value in colors.__dict__.items():
         if isinstance(value, Color):
             _namedColors[name] = value
 
@@ -711,18 +729,18 @@ def describe(aColor,mode=0):
     '''
     namedColors = getAllNamedColors()
     closest = (10, None, None)  #big number, name, color
-    for (name, color) in namedColors.items():
+    for name, color in namedColors.items():
         distance = colorDistance(aColor, color)
         if distance < closest[0]:
             closest = (distance, name, color)
     if mode<=1:
         s = 'best match is %s, distance %0.4f' % (closest[1], closest[0])
-        if mode==0: print s
+        if mode==0: print(s)
         else: return s
     elif mode==2:
         return (closest[1], closest[0])
     else:
-        raise ValueError, "Illegal value for mode "+str(mode)
+        raise ValueError("Illegal value for mode "+str(mode))
 
 def hue2rgb(m1, m2, h):
     if h<0: h += 1
@@ -808,7 +826,7 @@ class cssParse:
             if hsl:
                 R,G,B= hsl2rgb(self.hueVal(n[0]),self.pcVal(n[1]),self.pcVal(n[2]))
             else:
-                R,G,B = map('%' in n[0] and self.rgbPcVal or self.rgbVal,n)
+                R,G,B = list(map('%' in n[0] and self.rgbPcVal or self.rgbVal,n))
 
             return Color(R,G,B,a)
 
@@ -831,7 +849,8 @@ class toColor:
             assert 3<=len(arg)<=4, 'Can only convert 3 and 4 sequences to color'
             assert 0<=min(arg) and max(arg)<=1
             return len(arg)==3 and Color(arg[0],arg[1],arg[2]) or CMYKColor(arg[0],arg[1],arg[2],arg[3])
-        elif isinstance(arg,basestring):
+        elif isStr(arg):
+            arg = asNative(arg)
             C = cssParse(arg)
             if C: return C
             if arg in self.extraColorsNS: return self.extraColorsNS[arg]
@@ -867,7 +886,7 @@ def setColors(**kw):
         progress = 0
         for k, v in kw.items():
             if isinstance(v,(tuple,list)):
-                c = map(lambda x,UNDEF=UNDEF: toColor(x,UNDEF),v)
+                c = list(map(lambda x,UNDEF=UNDEF: toColor(x,UNDEF),v))
                 if isinstance(v,tuple): c = tuple(c)
                 ok = UNDEF not in c
             else:
@@ -992,8 +1011,8 @@ def _enforceRGB(c):
     return tc
 
 def _chooseEnforceColorSpace(enforceColorSpace):
-    if enforceColorSpace is not None and not callable(enforceColorSpace):
-        if isinstance(enforceColorSpace,basestring): enforceColorSpace=enforceColorSpace.upper()
+    if enforceColorSpace is not None and not isinstance(enforceColorSpace, collections.Callable):
+        if isinstance(enforceColorSpace,str): enforceColorSpace=enforceColorSpace.upper()
         if enforceColorSpace=='CMYK':
             enforceColorSpace = _enforceCMYK
         elif enforceColorSpace=='RGB':
