@@ -1,7 +1,7 @@
 #Copyright ReportLab Europe Ltd. 2000-2012
 #see license.txt for license details
 #history http://www.reportlab.co.uk/cgi-bin/viewcvs.cgi/public/reportlab/trunk/reportlab/platypus/flowables.py
-__version__=''' $Id: flowables.py 3959 2012-09-27 14:39:39Z robin $ '''
+__version__=''' $Id$ '''
 __doc__="""
 A flowable is a "floating element" in a document whose exact position is determined by the
 other elements that precede it, such as a paragraph, a diagram interspersed between paragraphs,
@@ -27,12 +27,13 @@ higher level components).
 import os
 from copy import deepcopy, copy
 from reportlab.lib.colors import red, gray, lightgrey
-from reportlab.lib.utils import fp_str
+from reportlab.lib.rl_accel import fp_str
 from reportlab.lib.enums import TA_LEFT, TA_CENTER, TA_RIGHT, TA_JUSTIFY
 from reportlab.lib.styles import _baseFontName
 from reportlab.pdfbase import pdfutils
 from reportlab.pdfbase.pdfmetrics import stringWidth
 from reportlab.rl_config import _FUZZ, overlapAttachedSpace, ignoreContainerActions
+import collections
 
 __all__=('TraceInfo','Flowable','XBox','Preformatted','Image','Spacer','PageBreak','SlowPageBreak',
         'CondPageBreak','KeepTogether','Macro','CallerMacro','ParagraphAndImage',
@@ -98,7 +99,7 @@ class Flowable:
             elif a in ('RIGHT',TA_RIGHT):
                 x += sW
             elif a not in ('LEFT',TA_LEFT):
-                raise ValueError, "Bad hAlign value "+str(a)
+                raise ValueError("Bad hAlign value "+str(a))
         return x
 
     def drawOn(self, canvas, x, y, _sW=0):
@@ -491,7 +492,7 @@ class Image(Flowable):
 
     def identity(self,maxLen=None):
         r = Flowable.identity(self,maxLen)
-        if r[-4:]=='>...' and isinstance(self.filename,basestring):
+        if r[-4:]=='>...' and isinstance(self.filename,str):
             r = "%s filename=%s>" % (r[:-4],self.filename)
         return r
 
@@ -545,7 +546,7 @@ class CondPageBreak(Spacer):
         if availHeight<self.height:
             f = self._doctemplateAttr('frame')
             if not f: return availWidth, availHeight
-            from doctemplate import FrameBreak
+            from reportlab.platypus.doctemplate import FrameBreak
             f.add_generated_content(FrameBreak)
         return 0, 0
 
@@ -557,7 +558,7 @@ def _listWrapOn(F,availWidth,canv,mergeSpace=1,obj=None,dims=None):
     doct = getattr(canv,'_doctemplate',None)
     cframe = getattr(doct,'frame',None)
     if cframe:
-        from reportlab.platypus.doctemplate import _addGeneratedContent
+        from reportlab.platypus.doctemplate import _addGeneratedContent, Indenter
         doct_frame = cframe
         cframe = doct.frame = deepcopy(doct_frame)
         cframe._generated_content = None
@@ -570,22 +571,32 @@ def _listWrapOn(F,availWidth,canv,mergeSpace=1,obj=None,dims=None):
         F = F[:]
         while F:
             f = F.pop(0)
-            if hasattr(f,'frameAction'): continue
+            if hasattr(f,'frameAction'):
+                from reportlab.platypus.doctemplate import Indenter
+                if isinstance(f,Indenter):
+                    availWidth -= f.left+f.right
+                continue
             w,h = f.wrapOn(canv,availWidth,0xfffffff)
             if dims is not None: dims.append((w,h))
             if cframe:
                 _addGeneratedContent(F,cframe)
             if w<=_FUZZ or h<=_FUZZ: continue
-            W = max(W,w)
+            W = max(W,min(w,availWidth))
             H += h
             if not atTop:
                 h = f.getSpaceBefore()
-                if mergeSpace: h = max(h-pS,0)
+                if mergeSpace:
+                    if getattr(f,'_SPACETRANSFER',False):
+                        h = pS
+                    h = max(h-pS,0)
                 H += h
             else:
                 if obj is not None: obj._spaceBefore = f.getSpaceBefore()
                 atTop = 0
-            pS = f.getSpaceAfter()
+            s = f.getSpaceAfter()
+            if getattr(f,'_SPACETRANSFER',False):
+                s = pS
+            pS = s
             H += pS
         if obj is not None: obj._spaceAfter = pS
         return W, H-pS
@@ -596,7 +607,7 @@ def _listWrapOn(F,availWidth,canv,mergeSpace=1,obj=None,dims=None):
 def _flowableSublist(V):
     "if it isn't a list or tuple, wrap it in a list"
     if not isinstance(V,(list,tuple)): V = V is not None and [V] or []
-    from doctemplate import LCActionFlowable
+    from reportlab.platypus.doctemplate import LCActionFlowable
     assert not [x for x in V if isinstance(x,LCActionFlowable)],'LCActionFlowables not allowed in sublists'
     return V
 
@@ -624,7 +635,7 @@ class KeepTogether(_ContainerSpace,Flowable):
 
     def __repr__(self):
         f = self._content
-        L = map(repr,f)
+        L = list(map(repr,f))
         L = "\n"+"\n".join(L)
         L = L.replace("\n", "\n  ")
         return "%s(%s,maxHeight=%s)" % (self.__class__.__name__,L,self._maxHeight)
@@ -646,10 +657,10 @@ class KeepTogether(_ContainerSpace,Flowable):
         C1 = (self._H0>aH) or C0 and atTop
         if C0 or C1:
             if C0:
-                from doctemplate import FrameBreak
+                from reportlab.platypus.doctemplate import FrameBreak
                 A = FrameBreak
             else:
-                from doctemplate import NullActionFlowable
+                from reportlab.platypus.doctemplate import NullActionFlowable
                 A = NullActionFlowable
             S.insert(0,A())
         return S
@@ -672,7 +683,7 @@ class Macro(Flowable):
     def wrap(self, availWidth, availHeight):
         return (0,0)
     def draw(self):
-        exec self.command in globals(), {'canvas':self.canv}
+        exec(self.command, globals(), {'canvas':self.canv})
 
 class CallerMacro(Flowable):
     '''
@@ -825,7 +836,9 @@ def cdeepcopy(obj):
 class _Container(_ContainerSpace):  #Abstract some common container like behaviour
     def drawOn(self, canv, x, y, _sW=0, scale=1.0, content=None, aW=None):
         '''we simulate being added to a frame'''
-        from doctemplate import ActionFlowable
+        from reportlab.platypus.doctemplate import ActionFlowable, Indenter
+        x0 = x
+        y0 = y
         pS = 0
         if aW is None: aW = self.width
         aW *= scale
@@ -833,18 +846,44 @@ class _Container(_ContainerSpace):  #Abstract some common container like behavio
             content = self._content
         x = self._hAlignAdjust(x,_sW*scale)
         y += self.height*scale
+        yt = y
+        frame = getattr(self,'_frame',None)
         for c in content:
             if not ignoreContainerActions and isinstance(c,ActionFlowable):
                 c.apply(self.canv._doctemplate)
                 continue
+            if isinstance(c,Indenter):
+                x += c.left*scale
+                aW -= (c.left+c.right)*scale
+                continue
             w, h = c.wrapOn(canv,aW,0xfffffff)
             if (w<_FUZZ or h<_FUZZ) and not getattr(c,'_ZEROSIZE',None): continue
-            if c is not content[0]: h += max(c.getSpaceBefore()-pS,0)
+            if yt!=y:
+                s = c.getSpaceBefore()
+                if not getattr(c,'_SPACETRANSFER',False):
+                    h += max(s-pS,0)
             y -= h
+            fbg = getattr(frame,'_frameBGs',None)
+            s = c.getSpaceAfter()
+            if getattr(c,'_SPACETRANSFER',False):
+                s = pS
+            pS = s
+            if fbg:
+                fbgl, fbgr, fbgc = fbg[-1]
+                fbw = scale*(frame._width-fbgl-fbgr)
+                fbh = y + h + pS
+                fby = max(y0,y-pS)
+                fbh = max(0,fbh-fby)
+                if abs(fbw)>_FUZZ and abs(fbh)>_FUZZ:
+                    canv.saveState()
+                    canv.setFillColor(fbgc)
+                    canv.rect(x0+scale*(fbgl-frame._leftPadding)-0.1,fby-0.1,fbw+0.2,fbh+0.2,stroke=0,fill=1)
+                    canv.restoreState()
+            c._frame = frame
             c.drawOn(canv,x,y,_sW=aW-w)
-            if c is not content[-1]:
-                pS = c.getSpaceAfter()
+            if c is not content[-1] and not getattr(c,'_SPACETRANSFER',None):
                 y -= pS
+            del c._frame
 
     def copyContent(self,content=None):
         C = [].append
@@ -874,16 +913,18 @@ class PTOContainer(_Container,Flowable):
         return self.width,self.height
 
     def split(self, availWidth, availHeight):
+        from reportlab.platypus.doctemplate import Indenter
         if availHeight<0: return []
         canv = self.canv
         C = self._content
         x = i = H = pS = hx = 0
         n = len(C)
         I2W = {}
-        for x in xrange(n):
+        dLeft = dRight = 0
+        for x in range(n):
             c = C[x]
             I = c._ptoinfo
-            if I not in I2W.keys():
+            if I not in list(I2W.keys()):
                 T = I.trailer
                 Hdr = I.header
                 tW, tH = _listWrapOn(T, availWidth, self.canv)
@@ -895,10 +936,18 @@ class PTOContainer(_Container,Flowable):
             else:
                 T,tW,tH,tSB = I2W[I]
             _, h = c.wrapOn(canv,availWidth,0xfffffff)
-            if x:
-                hx = max(c.getSpaceBefore()-pS,0)
-                h += hx
-            pS = c.getSpaceAfter()
+            if isinstance(c,Indenter):
+                dw = c.left+c.right
+                dLeft += c.left
+                dRight += c.right
+                availWidth -= dw
+                pS = 0
+                hx = 0
+            else:
+                if x:
+                    hx = max(c.getSpaceBefore()-pS,0)
+                    h += hx
+                pS = c.getSpaceAfter()
             H += h+pS
             tHS = tH+max(tSB,pS)
             if H+tHS>=availHeight-_FUZZ: break
@@ -914,6 +963,12 @@ class PTOContainer(_Container,Flowable):
         else:
             SS = []
 
+        if abs(dLeft)+abs(dRight)>1e-8:
+            R1I = [Indenter(-dLeft,-dRight)]
+            R2I = [Indenter(dLeft,dRight)]
+        else:
+            R1I = R2I = []
+
         if not SS:
             j = i
             while i>1 and C[i-1].getKeepWithNext():
@@ -928,13 +983,13 @@ class PTOContainer(_Container,Flowable):
         F = [UseUpSpace()]
 
         if len(SS)>1:
-            R1 = C[:i] + SS[:1] + T + F
-            R2 = Hdr + SS[1:]+C[i+1:]
+            R1 = C[:i]+SS[:1]+R1I+T+F
+            R2 = Hdr+R2I+SS[1:]+C[i+1:]
         elif not i:
             return []
         else:
-            R1 = C[:i]+T+F
-            R2 = Hdr + C[i:]
+            R1 = C[:i]+R1I+T+F
+            R2 = Hdr+R2I+C[i:]
         T =  R1 + [PTOContainer(R2,[copy(x) for x in I.trailer],[copy(x) for x in I.header])]
         return T
 
@@ -1003,7 +1058,7 @@ class KeepInFrame(_Container,Flowable):
                 getattr(self,'maxHeight','')and (' maxHeight=%s' % fp_str(getattr(self,'maxHeight')))or '')
 
     def wrap(self,availWidth,availHeight):
-        from doctemplate import LayoutError
+        from reportlab.platypus.doctemplate import LayoutError
         mode = self.mode
         maxWidth = float(min(self.maxWidth or availWidth,availWidth))
         maxHeight = float(min(self.maxHeight or availHeight,availHeight))
@@ -1190,7 +1245,8 @@ class ImageAndFlowables(_Container,Flowable):
         if self._C0:
             _Container.drawOn(self, canv, Fx, y, content=self._C0, aW=self._iW)
         if self._C1:
-            _Container.drawOn(self, canv, x, y-self._aH,content=self._C1)
+            aW, aH = self._wrapArgs
+            _Container.drawOn(self, canv, x, y-self._aH,content=self._C1, aW=aW)
 
     def _findSplit(self,canv,availWidth,availHeight,mergeSpace=1,obj=None):
         '''return max width, required height for a list of flowables F'''
@@ -1214,7 +1270,7 @@ class ImageAndFlowables(_Container,Flowable):
                 return W, availHeight, F[:i],F[i:]
             H += h
             if H>availHeight:
-                from paragraph import Paragraph
+                from reportlab.platypus.paragraph import Paragraph
                 aH = availHeight-(H-h)
                 if isinstance(f,(Paragraph,Preformatted)):
                     leading = f.style.leading
@@ -1235,6 +1291,7 @@ class ImageAndFlowables(_Container,Flowable):
 class AnchorFlowable(Spacer):
     '''create a bookmark in the pdf'''
     _ZEROSIZE=1
+    _SPACETRANSFER = True
     def __init__(self,name):
         Spacer.__init__(self,0,0)
         self._name = name
@@ -1248,6 +1305,32 @@ class AnchorFlowable(Spacer):
     def draw(self):
         self.canv.bookmarkHorizontal(self._name,0,0)
 
+class FrameBG(AnchorFlowable):
+    """Start or stop coloring the frame background
+    left & right are distances from the edge of the frame to start stop colouring.
+    """
+    _ZEROSIZE=1
+    def __init__(self, color=None, left=0, right=0, start=True):
+        Spacer.__init__(self,0,0)
+        self.start = start
+        if start:
+            from reportlab.platypus.doctemplate import _evalMeasurement
+            self.left = _evalMeasurement(left)
+            self.right = _evalMeasurement(right)
+            self.color = color
+
+    def __repr__(self):
+        return "%s(%s)" % (self.__class__.__name__,', '.join(['%s=%r' % (i,getattr(self,i,None)) for i in 'start color left right'.split()]))
+
+    def draw(self):
+        frame = getattr(self,'_frame',None)
+        if frame is None: return
+        if self.start:
+            w = getattr(frame,'_lineWidth',0)
+            frame._frameBGs.append((self.left,self.right,self.color))
+        elif frame._frameBGs:
+            frame._frameBGs.pop()
+
 class FrameSplitter(NullDraw):
     '''When encountered this flowable should either switch directly to nextTemplate
     if remaining space in the current frame is less than gap+required or it should
@@ -1302,11 +1385,12 @@ class FrameSplitter(NullDraw):
 
 from reportlab.lib.sequencer import _type2formatter
 _bulletNames = dict(
-                circle=u'\u25cf',
-                square=u'\u25a0',
-                disc=u'\u25cf',
-                diamond=u'\u25c6',
-                rarrowhead=u'\u27a4',
+                bulletchar='\u2022',    #usually a small circle
+                circle='\u25cf',    #circle as high as the font
+                square='\u25a0',
+                disc='\u25cf',
+                diamond='\u25c6',
+                rarrowhead='\u27a4',
                 )
 
 def _bulletFormat(value,type='1',format=None):
@@ -1316,9 +1400,9 @@ def _bulletFormat(value,type='1',format=None):
         s = _type2formatter[type](int(value))
 
     if format:
-        if isinstance(format,basestring):
+        if isinstance(format,str):
             s = format % s
-        elif callable(format):
+        elif isinstance(format, collections.Callable):
             s = format(s)
         else:
             raise ValueError('unexpected BulletDrawer format %r' % format)
@@ -1538,13 +1622,13 @@ class ListFlowable(_Container,Flowable):
                 raise ValueError('%s style argument not a ListStyle' % self.__class__.__name__)
             self.style = style
 
-        for k,v in ListStyle.defaults.iteritems():
+        for k,v in ListStyle.defaults.items():
             setattr(self,'_'+k,kwds.get(k,getattr(style,k,v)))
         if start is None:
             start = getattr(self,'_start',None)
             if start is None:
                 if getattr(self,'_bulletType','1')=='bullet':
-                    start = 'circle'
+                    start = 'bulletchar'
                 else:
                     start = '1'
         self._start = start
@@ -1799,7 +1883,7 @@ class DocPara(DocAssign):
     def func(self):
         expr = self.expr
         if expr:
-            if not isinstance(expr,(str,unicode)): expr = str(expr)
+            if not isinstance(expr,str): expr = str(expr)
             return self._doctemplateAttr('docEval')(expr)
 
     def add_content(self,*args):
