@@ -1,4 +1,4 @@
-#Copyright ReportLab Europe Ltd. 2000-2004
+#Copyright ReportLab Europe Ltd. 2000-2012
 #see license.txt for license details
 """
 Tests for RLG Image shapes.
@@ -13,8 +13,8 @@ from reportlab.lib.pagesizes import A4
 
 
 IMAGES = []
-IMAGENAME = 'cs_logo.gif'
 IMAGENAME = 'pythonpowered.gif'
+GSIMAGE = 'pythonpowered-gs.gif'
 
 
 class ImageTestCase(unittest.TestCase):
@@ -76,8 +76,16 @@ class ImageTestCase(unittest.TestCase):
         d.rotate(45)
         IMAGES.append(d)
 
-        IMAGES.append(None) # used to indicate last test
 
+    def test4(self):
+        "Test convert a greyscale bitmap file as Image shape into a tmp. PDF file."
+
+        d = Drawing(110, 44)
+        img = Image(0, 0, 110, 44, GSIMAGE)
+        d.add(img)
+        d.translate(0,2*72)
+        IMAGES.append(d)
+        IMAGES.append(None) # used to indicate last test
 
 def makeSuite():
     return makeSuiteForClasses(ImageTestCase)
