@@ -1,17 +1,17 @@
 #!/bin/env python
-#Copyright ReportLab Europe Ltd. 2000-2008
+#Copyright ReportLab Europe Ltd. 2000-2012
 #see license.txt for license details
 __doc__='testscript for reportlab.pdfgen'
-__version__=''' $Id: test_pdfgen_general.py 3791 2010-09-29 19:37:05Z andy $ '''
+__version__=''' $Id$ '''
 #tests and documents new low-level canvas
 from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, printLocation
 setOutDir(__name__)
-import os, string
+import os
 import unittest
 from reportlab.pdfgen import canvas   # gmcm 2000/10/13, pdfgen now a package
 from reportlab.lib.units import inch, cm
 from reportlab.lib import colors
-from reportlab.lib.utils import haveImages, fileName2Utf8
+from reportlab.lib.utils import haveImages, fileName2FSEnc
 
 #################################################################
 #
@@ -83,9 +83,8 @@ def makesubsection(canvas, title, horizontal):
 
 
 #def newsubsection(name):
-#    from types import TupleType
 #    thissection = outlinenametree[-1]
-#    if type(thissection) is not TupleType:
+#    if not isinstance(thissection,tuple):
 #        subsectionlist = []
 #        thissection = outlinenametree[-1] = (thissection, subsectionlist)
 #    else:
@@ -106,9 +105,9 @@ class DocBlock:
 
     def _getHeight(self):
         "splits into lines"
-        self.comment1lines = string.split(self.comment1, '\n')
-        self.codelines = string.split(self.code, '\n')
-        self.comment2lines = string.split(self.comment2, '\n')
+        self.comment1lines = self.comment1.split('\n')
+        self.codelines = self.code.split('\n')
+        self.comment2lines = self.comment2.split('\n')
         textheight = (len(self.comment1lines) +
                 len(self.code) +
                 len(self.comment2lines) +
@@ -133,7 +132,7 @@ class DocBlock:
         self.namespace = {'canvas':canvas,'cm': cm,'inch':inch}
         canvas.translate(x+9, y - height + 9)
         codeObj = compile(self.code, '<sample>','exec')
-        exec codeObj in self.namespace
+        exec(codeObj, self.namespace)
 
         canvas.restoreState()
 
@@ -204,7 +203,7 @@ def makeDocument(filename, pageCallBack=None):
 
     #quickie encoding test: when canvas encoding not set,
     #the following should do (tm), (r) and (c)
-    msg_uni = u'copyright\u00A9 trademark\u2122 registered\u00AE ReportLab in unicode!'
+    msg_uni = 'copyright\u00A9 trademark\u2122 registered\u00AE scissors\u2702: ReportLab in unicode!'
     msg_utf8 = msg_uni.replace('unicode','utf8').encode('utf8')
     c.drawString(100, 100, msg_uni)
     c.drawString(100, 80, msg_utf8)
@@ -434,43 +433,54 @@ cost to performance.""")
     t.textLine()
 
     #line demo
-    makesubsection(c, "lines", 10*inch)
-    c.line(inch, 8*inch, 3*inch, 8*inch)
-    t.setTextOrigin(4*inch, 8*inch)
+    makesubsection(c, "lines", 9*inch)
+    c.line(inch, 9*inch, 3*inch, 9*inch)
+    t.setTextOrigin(4*inch, 9*inch)
     t.textLine('canvas.line(x1, y1, x2, y2)')
 
     #bezier demo - show control points
-    makesubsection(c, "bezier curves", 7.5*inch)
+    makesubsection(c, "bezier curves", 8.5*inch)
     (x1, y1, x2, y2, x3, y3, x4, y4) = (
-                        inch, 6.5*inch,
-                        1.2*inch, 7.5 * inch,
-                        3*inch, 7.5 * inch,
-                        3.5*inch, 6.75 * inch
+                        inch, 7.8*inch,
+                        1.2*inch, 8.8 * inch,
+                        3*inch, 8.8 * inch,
+                        3.5*inch, 8.05 * inch
                         )
     c.bezier(x1, y1, x2, y2, x3, y3, x4, y4)
     c.setDash(3,3)
     c.line(x1,y1,x2,y2)
     c.line(x3,y3,x4,y4)
     c.setDash()
-    t.setTextOrigin(4*inch, 7 * inch)
+    t.setTextOrigin(4*inch, 8.3 * inch)
     t.textLine('canvas.bezier(x1, y1, x2, y2, x3, y3, x4, y4)')
 
     #rectangle
-    makesubsection(c, "rectangles", 7*inch)
-    c.rect(inch, 5.25 * inch, 2 * inch, 0.75 * inch)
-    t.setTextOrigin(4*inch, 5.5 * inch)
+    makesubsection(c, "rectangles", 8*inch)
+    c.rect(inch, 7 * inch, 2 * inch, 0.75 * inch)
+    t.setTextOrigin(4*inch, 7.375 * inch)
     t.textLine('canvas.rect(x, y, width, height) - x,y is lower left')
 
+    c.roundRect(inch,6.25*inch,2*inch,0.6*inch,0.1*inch)
+    t.setTextOrigin(4*inch, 6.55*inch)
+    t.textLine('canvas.roundRect(x,y,width,height,radius)')
+
+    makesubsection(c, "arcs", 8*inch)
+    c.arc(inch,5*inch,3*inch,6*inch,0,90)
+    t.setTextOrigin(4*inch, 5.5*inch)
+    t.textLine('canvas.arc(x1, y1, x2, y2, startDeg, extentDeg)')
+    t.textLine('Note that this is an elliptical arc, not just circular!')
+
+
     #wedge
     makesubsection(c, "wedges", 5*inch)
-    c.wedge(inch, 5*inch, 3*inch, 4*inch, 0, 315)
-    t.setTextOrigin(4*inch, 4.5 * inch)
+    c.wedge(inch, 4.5*inch, 3*inch, 3.5*inch, 0, 315)
+    t.setTextOrigin(4*inch, 4*inch)
     t.textLine('canvas.wedge(x1, y1, x2, y2, startDeg, extentDeg)')
     t.textLine('Note that this is an elliptical arc, not just circular!')
 
     #wedge the other way
-    c.wedge(inch, 4*inch, 3*inch, 3*inch, 0, -45)
-    t.setTextOrigin(4*inch, 3.5 * inch)
+    c.wedge(inch, 3.75*inch, 3*inch, 2.75*inch, 0, -45)
+    t.setTextOrigin(4*inch, 3*inch)
     t.textLine('Use a negative extent to go clockwise')
 
     #circle
@@ -669,8 +679,10 @@ cost to performance.""")
         from reportlab.lib.testutils import testsFolder
         gif = os.path.join(testsFolder,'pythonpowered.gif')
         c.drawInlineImage(gif,2*inch, 7*inch)
+        c.drawInlineImage(os.path.join(testsFolder,'pythonpowered-gs.gif'),4*inch, 7.5*inch)
     else:
         c.rect(2*inch, 7*inch, 110, 44)
+        c.rect(4*inch, 7*inch, 110, 44)
 
     c.line(1.5*inch, 7*inch, 4*inch, 7*inch)
     c.line(2*inch, 6.5*inch, 2*inch, 8*inch)
@@ -713,7 +725,7 @@ cost to performance.""")
     c.drawString(1*inch, 10.25*inch-2*14.4, "The second image is white alpha=0% to purple=100%")
 
 
-    for i in xrange(8):
+    for i in range(8):
         c.drawString(1*inch,8*inch+i*14.4,"mask=None   Line %d"%i)
         c.drawString(3*inch,8*inch+i*14.4,"mask='auto' Line %d"%i)
         c.drawString(1*inch,6*inch+i*14.4,"mask=None   Line %d"%i)
@@ -855,11 +867,40 @@ cost to performance.""")
                       color=colors.magenta)
     c.drawString(inch+3, 2*inch+6, 'Hyperlink with custom border style')
 
-    xpdf = fileName2Utf8(outputfile('test_hello.pdf').replace('\\','/'))
+    xpdf = fileName2FSEnc(outputfile('test_hello.pdf').replace('\\','/'))
     link = 'Hard link to %s, with red border' % xpdf
     r1 = (inch, 1.5*inch, inch+2*3+c.stringWidth(link,c._fontname, c._fontsize), 1.75*inch) # this is x1,y1,x2,y2
     c.linkURL(xpdf, r1, thickness=1, color=colors.red, kind='GoToR')
     c.drawString(inch+3, 1.5*inch+6, link )
+    c.showPage()
+
+    ############# colour gradients
+    title = 'Gradients code contributed by Peter Johnson <johnson.peter@gmail.com>'
+    c.drawString(1*inch,10.8*inch,title)
+    c.addOutlineEntry(title+" section", title, level=0, closed=True)
+    c.bookmarkHorizontalAbsolute(title, 10.8*inch)
+    from reportlab.lib.colors import red, green, blue
+
+    c.saveState()
+    p = c.beginPath()
+    p.moveTo(1*inch,2*inch)
+    p.lineTo(1.5*inch,2.5*inch)
+    p.curveTo(2*inch,3*inch,3.0*inch,3*inch,4*inch,2.9*inch)
+    p.lineTo(5.5*inch,2.1*inch)
+    p.close()
+    c.clipPath(p)
+
+    # Draw a linear gradient from (0, 2*inch) to (5*inch, 3*inch), from orange to white.
+    # The gradient will extend past the endpoints (so you probably want a clip path in place)
+    c.linearGradient(1*inch, 2*inch, 6*inch, 3*inch, (red, blue))
+    c.restoreState()
+
+    # Draw a radial gradient with a radius of 3 inches.
+    # The color starts orange and stays orange until 20% of the radius,
+    # then fades to white at 80%, and ends up green at 3 inches from the center.
+    # Since extend is false, the gradient stops drawing at the edge of the circle.
+    c.radialGradient(4*inch, 6*inch, 3*inch, (red, green, blue), (0.2, 0.8, 1.0), extend=False)
+    c.showPage()
 
     ### now do stuff for the outline
     #for x in outlinenametree: print x
@@ -870,10 +911,10 @@ cost to performance.""")
 
 def run(filename):
     c = makeDocument(filename)
-    c.setAuthor(u'R\xfcp\xe9rt B\xe8\xe4r')
+    c.setAuthor('R\xfcp\xe9rt B\xe8\xe4r')
     c.setTitle('R\xc3\xbcp\xc3\xa9rt B\xc3\xa8\xc3\xa4r\'s Book')
-    c.setCreator(u'Some Creator')
-    c.setSubject(u'Some Subject')
+    c.setCreator('Some Creator')
+    c.setSubject('Some Subject')
     c.save()
     c = makeDocument(filename)
     import os
@@ -896,7 +937,7 @@ def pageShapes(c):
     d = DocBlock()
     d.comment1 = 'Lesson one'
     d.code = "canvas.textOut('hello, world')"
-    print d.code
+    print(d.code)
 
     d.comment2 = 'Lesson two'
 
@@ -926,7 +967,7 @@ class PdfgenTestCase(unittest.TestCase):
         c.addPageLabel(102, prefix="Back",start=1)
 
         # Make some (mostly) empty pages
-        for i in xrange(113):
+        for i in range(113):
             c.drawString(100, 100, 'Tis is page '+str(i))
             c.showPage()
 
@@ -1013,11 +1054,54 @@ class PdfgenTestCase(unittest.TestCase):
         trySomeColors(rgb+cmykb,'rgb')
         self.assertRaises(ValueError,trySomeColors,cmyk+rgb+seps,'cmyk')
         trySomeColors(cmyk+['black']+seps,'cmyk')   #OK because black & seps are convertible
-        
+
+    def test5(self):
+        from reportlab.lib.pagesizes import A4,LETTER
+        canv = canvas.Canvas(outputfile('test_pdfgen_general_page_sizes.pdf'),
+                        pagesize=A4,
+                        )
+        canv.setFont('Helvetica',10)
+        S = A4
+        canv.drawString(0,S[1]-10,'Top Left=(%s,%s) Page Size=%s x %s' % (0,S[1],S[0],S[1]))
+        canv.drawCentredString(0.5*S[0],0.5*S[1],'Center =(%s,%s) Page Size=%s x %s' % (0.5*S[0],0.5*S[1],S[0],S[1]))
+        canv.drawRightString(S[0],2,'Bottom Right=(%s,%s) Page Size=%s x %s' % (S[0],0,S[0],S[1]))
+        canv.showPage()
+        S = LETTER
+        canv.setPageSize(S)
+        canv.drawString(0,S[1]-10,'Top Left=(%s,%s) Page Size=%s x %s' % (0,S[1],S[0],S[1]))
+        canv.drawCentredString(0.5*S[0],0.5*S[1],'Center =(%s,%s) Page Size=%s x %s' % (0.5*S[0],0.5*S[1],S[0],S[1]))
+        canv.drawRightString(S[0],2,'Bottom Right=(%s,%s) Page Size=%s x %s' % (S[0],0,S[0],S[1]))
+        canv.showPage()
+        S = A4
+        canv.setPageSize(S)
+        canv.setPageRotation(180)
+        canv.drawString(0,S[1]-10,'Top Left=(%s,%s) Page Size=%s x %s' % (0,S[1],S[0],S[1]))
+        canv.drawCentredString(0.5*S[0],0.5*S[1],'Center =(%s,%s) Page Size=%s x %s' % (0.5*S[0],0.5*S[1],S[0],S[1]))
+        canv.drawRightString(S[0],2,'Bottom Right=(%s,%s) Page Size=%s x %s' % (S[0],0,S[0],S[1]))
+        canv.showPage()
+        S = A4[1],A4[0]
+        canv.setPageSize(S)
+        canv.setPageRotation(0)
+        canv.drawString(0,S[1]-30,'Top Left=(%s,%s) Page Size=%s x %s' % (0,S[1],S[0],S[1]))
+        canv.drawCentredString(0.5*S[0],0.5*S[1],'Center =(%s,%s) Page Size=%s x %s' % (0.5*S[0],0.5*S[1],S[0],S[1]))
+        canv.drawRightString(S[0],32,'Bottom Right=(%s,%s) Page Size=%s x %s' % (S[0],0,S[0],S[1]))
+        canv.showPage()
+        canv.save()
+
+    def testMultipleSavesOk(self):
+        c=canvas.Canvas(outputfile('test_pdfgen_savetwice.pdf'))
+        c.drawString(100, 700, 'Hello. This was saved twice')
+        c.showPage()
+
+        # Output the PDF
+        stuff = c.getpdfdata()
+        #multiple calls to save / getpdfdata used to cause errors
+        stuff = c.getpdfdata()    
+
 
 def trySomeColors(C,enforceColorSpace=None):
-    from StringIO import StringIO
-    out=StringIO()
+    from reportlab.lib.utils import getBytesIO
+    out=getBytesIO()
     canv = canvas.Canvas(out,enforceColorSpace=enforceColorSpace)
     canv.setFont('Helvetica',10)
     x = 0
