@@ -6,9 +6,8 @@
 from reportlab.lib.testutils import setOutDir,makeSuiteForClasses, outputfile, SecureTestCase, GlobDirectoryWalker, printLocation
 setOutDir(__name__)
 from reportlab.lib.testutils import RL_HOME,testsFolder
-__version__=''' $Id: test_source_chars.py 3959 2012-09-27 14:39:39Z robin $ '''
-import os, sys, glob, string, re
-from types import ModuleType, ClassType, MethodType, FunctionType
+__version__=''' $Id$ '''
+import os, sys, glob, re
 import reportlab
 import unittest
 from reportlab.lib.utils import open_and_read
@@ -26,7 +25,7 @@ class SourceTester(SecureTestCase):
 
     def checkFileForTabs(self, filename):
         txt = open_and_read(filename, 'r')
-        chunks = string.split(txt, '\t')
+        chunks = txt.split('\t')
         tabCount = len(chunks) - 1
         if tabCount:
             #raise Exception, "File %s contains %d tab characters!" % (filename, tabCount)
@@ -37,8 +36,8 @@ class SourceTester(SecureTestCase):
         initSize = len(txt)
         badLines = 0
         badChars = 0
-        for line in string.split(txt, '\n'):
-            stripped = string.rstrip(line)
+        for line in txt.split('\n'):
+            stripped = line.rstrip()
             spaces = len(line) - len(stripped)  # OK, so they might be trailing tabs, who cares?
             if spaces:
                 badLines = badLines + 1
@@ -57,10 +56,10 @@ def zapTrailingWhitespace(dirname):
     """Eliminates trailing spaces IN PLACE.  Use with extreme care
     and only after a backup or with version-controlled code."""
     assert os.path.isdir(dirname), "Directory not found!"
-    print "This will eliminate all trailing spaces in py files under %s." % dirname
-    ok = raw_input("Shall I proceed?  type YES > ")
+    print("This will eliminate all trailing spaces in py files under %s." % dirname)
+    ok = input("Shall I proceed?  type YES > ")
     if ok != 'YES':
-        print 'aborted by user'
+        print('aborted by user')
         return
     w = GlobDirectoryWalker(dirname, '*.py')
     for filename in w:
@@ -68,17 +67,17 @@ def zapTrailingWhitespace(dirname):
         txt = open(filename, 'r').read()
         badChars = 0
         cleaned = []
-        for line in string.split(txt, '\n'):
-            stripped = string.rstrip(line)
+        for line in txt.split('\n'):
+            stripped = line.rstrip()
             cleaned.append(stripped)
             spaces = len(line) - len(stripped)  # OK, so they might be trailing tabs, who cares?
             if spaces:
                 badChars = badChars + spaces
 
         if badChars != 0:
-            open(filename, 'w').write(string.join(cleaned, '\n'))
-            print "file %s contained %d trailing spaces, FIXED" % (filename, badChars)
-    print 'done'
+            open(filename, 'w').write('\n'.join(cleaned))
+            print("file %s contained %d trailing spaces, FIXED" % (filename, badChars))
+    print('done')
 
 def makeSuite():
     return makeSuiteForClasses(SourceTester)
